@@ -12,13 +12,18 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-import StringIO
+
+import errno
+import ferry
 import os
 import sys
 import json
 import logging
 import requests
 import re
+import shutil
+import StringIO
+from requests.exceptions import ConnectionError
 from subprocess import Popen, PIPE
 from prettytable import *
 from ferry.options import CmdHelp
@@ -28,7 +33,7 @@ class CLI(object):
     def __init__(self):
         self.cmds = CmdHelp()
         self.cmds.description = "Development environment for big data applications"
-        self.cmds.version = "0.1"
+        self.cmds.version = ferry.__version__
         self.cmds.usage = "ferry COMMAND [arg...]"
         self.cmds.add_option("-m", "--mode", "Deployment mode")
         self.cmds.add_option("-c", "--conf", "Deployment configuration")
@@ -61,8 +66,13 @@ class CLI(object):
         payload = { 'payload' : json.dumps(stack_description),
                     'mode' : mode, 
                     'conf' : conf }
-        res = requests.post(self.ferry_server + '/create', data=payload)
-        return str(res.text)
+
+        try:
+            res = requests.post(self.ferry_server + '/create', data=payload)
+            return str(res.text)
+        except ConnectionError:
+            logging.error("could not connect to ferry server")
+            return "It appears Ferry servers are not running.\nType sudo ferry server and try again."
 
     def _format_snapshots_query(self, json_data):
         bases = []
@@ -80,9 +90,8 @@ class CLI(object):
         t.add_column("Date", date)
         return t.get_string(sortby="Date",
                             border=True, 
-                            vrules=ALL, 
-                            vertical_char=' ', 
-                            junction_char=' ', 
+                            hrules=FRAME, 
+                            vrules=NONE, 
                             padding_width=4)
 
     def _format_table_query(self, json_data):
@@ -125,11 +134,22 @@ class CLI(object):
 
         return t.get_string(sortby="UUID",
                             border=True, 
-                            vrules=ALL, 
-                            vertical_char=' ', 
-                            junction_char=' ', 
+                            hrules=FRAME, 
+                            vrules=NONE, 
                             padding_width=4)
 
+    def _stop_all(self):
+        try:
+            constraints = { 'status' : 'running' }
+            payload = { 'constraints' : json.dumps(constraints) }
+            res = requests.get(self.ferry_server + '/query', params=payload)
+            stacks = json.loads(res.text)
+            for uuid in stacks.keys():
+                self._manage_stacks({'uuid' : uuid,
+                                     'action' : 'stop'})
+        except ConnectionError:
+            logging.error("could not connect to ferry server")
+        
     def _read_stacks(self, show_all=False, args=None):
         try:
             res = requests.get(self.ferry_server + '/query')
@@ -147,14 +167,18 @@ class CLI(object):
 
             # Merge the replies and format.
             return self._format_table_query(dict(query_reply.items() + deployed_reply.items()))
-        except requests.ConnectionError:
+        except ConnectionError:
             logging.error("could not connect to ferry server")
-            return 'could not connect to ferry server'
+            return "It appears Ferry servers are not running.\nType sudo ferry server and try again."
 
     def _list_snapshots(self):
-        res = requests.get(self.ferry_server + '/snapshots')
-        json_reply = json.loads(res.text)
-        return self._format_snapshots_query(json_reply)
+        try:
+            res = requests.get(self.ferry_server + '/snapshots')
+            json_reply = json.loads(res.text)
+            return self._format_snapshots_query(json_reply)
+        except ConnectionError:
+            logging.error("could not connect to ferry server")
+            return "It appears Ferry servers are not running.\nType sudo ferry server and try again."
 
     def _format_stack_inspect(self, json_data):
         return json.dumps(json_data, 
@@ -166,19 +190,34 @@ class CLI(object):
     """
     def _inspect_stack(self, stack_id):
         payload = { 'uuid':stack_id }
-        res = requests.get(self.ferry_server + '/stack', params=payload)
-        json_value = json.loads(str(res.text))
-        return self._format_stack_inspect(json_value)
-
+        try:
+            res = requests.get(self.ferry_server + '/stack', params=payload)
+            json_value = json.loads(str(res.text))
+            return self._format_stack_inspect(json_value)
+        except ConnectionError:
+            logging.error("could not connect to ferry server")
+            return "It appears Ferry servers are not running.\nType sudo ferry server and try again."
     """
     Copy over the logs. 
     """
     def _copy_logs(self, stack_id, to_dir):
         payload = {'uuid':stack_id,
                    'dir':to_dir}
-        res = requests.get(self.ferry_server + '/logs', params=payload)
-        json_value = json.loads(str(res.text))
-        return self._format_stack_inspect(json_value)
+        try:
+            res = requests.get(self.ferry_server + '/logs', params=payload)
+            json_value = json.loads(str(res.text))
+            return self._format_stack_inspect(json_value)
+        except ConnectionError:
+            logging.error("could not connect to ferry server")
+            return "It appears Ferry servers are not running.\nType sudo ferry server and try again."
+
+    """
+    Read the location of the directory containing the keys
+    used to communicate with the containers. 
+    """
+    def _read_key_dir(self):
+        f = open(ferry.install.DEFAULT_DOCKER_KEY, 'r')
+        return f.read().strip()
 
     """
     Connector a specific client/connector via ssh. 
@@ -186,8 +225,12 @@ class CLI(object):
     def _connect_stack(self, stack_id, connector_id):
         # Get the IP and default user information for this connector.
         payload = {'uuid':stack_id}
-        res = requests.get(self.ferry_server + '/stack', params=payload)
-        json_value = json.loads(str(res.text))
+        try:
+            res = requests.get(self.ferry_server + '/stack', params=payload)
+            json_value = json.loads(str(res.text))
+        except ConnectionError:
+            logging.error("could not connect to ferry server")
+            return "It appears Ferry servers are not running.\nType sudo ferry server and try again."
 
         connector_ip = None
         for cg in json_value['connectors']:
@@ -202,9 +245,13 @@ class CLI(object):
 
         # Now form the ssh command. This just executes in the same shell. 
         if connector_ip:
+            key_opt = '-o StrictHostKeyChecking=no'
+            host_opt = '-o UserKnownHostsFile=/dev/null'
+            ident = '-i %s/id_rsa' % self._read_key_dir()
             dest = '%s@%s' % (self.default_user, connector_ip)
-            logging.warning("ssh %s" % dest)
-            os.execv('/usr/bin/ssh', ['', dest])
+            cmd = "ssh %s %s %s %s" % (key_opt, host_opt, ident, dest)
+            logging.warning(cmd)
+            os.execv('/usr/bin/ssh', cmd.split())
 
     def _parse_deploy_arg(self, param, args, default):
         pattern = re.compile('--%s=(\w+)' % param)
@@ -224,15 +271,23 @@ class CLI(object):
         payload = { 'uuid' : stack_id,
                     'mode' : mode,
                     'conf' : conf }
-        res = requests.post(self.ferry_server + '/deploy', data=payload)
-        return res.text
+        try:
+            res = requests.post(self.ferry_server + '/deploy', data=payload)
+            return res.text
+        except ConnectionError:
+            logging.error("could not connect to ferry server")
+            return "It appears Ferry servers are not running.\nType sudo ferry server and try again."
 
     """
     Manage the stack. 
     """
     def _manage_stacks(self, stack_info):
-        res = requests.post(self.ferry_server + '/manage/stack', data=stack_info)
-        return str(res.text)        
+        try:
+            res = requests.post(self.ferry_server + '/manage/stack', data=stack_info)
+            return str(res.text)        
+        except ConnectionError:
+            logging.error("could not connect to ferry server")
+            return "It appears Ferry servers are not running.\nType sudo ferry server and try again."
         
     """
     Output the help message.
@@ -244,13 +299,16 @@ class CLI(object):
     Output version information.
     """
     def _print_info(self):
-        res = requests.get(self.ferry_server + '/version')
+        try:
+            res = requests.get(self.ferry_server + '/version')
+            s = self.cmds.description + '\n'
+            s += "Version: %s\n" % self.cmds.version
+            s += "Docker: %s\n" % res.text.strip()
 
-        s = self.cmds.description + '\n'
-        s += "Version: %s\n" % self.cmds.version
-        s += "Docker: %s\n" % res.text.strip()
-
-        return s
+            return s
+        except ConnectionError:
+            logging.error("could not connect to ferry server")
+            return "It appears Ferry servers are not running.\nType sudo ferry server and try again."
         
     """
     Helper method to read a file.
@@ -264,31 +322,55 @@ class CLI(object):
 
         return json_text
 
+    def _check_ssh_key(self):
+        keydir = open(ferry.install.DEFAULT_DOCKER_KEY, 'r').read().strip()
+        if keydir == ferry.install.DEFAULT_KEY_DIR:
+            try:
+                ferry.install.GLOBAL_KEY_DIR = os.environ['HOME'] + '/.ssh/ferry'
+                os.makedirs(os.environ['HOME'] + '/.ssh/ferry')
+                shutil.copy(ferry.install.DEFAULT_KEY_DIR + '/id_rsa',
+                            ferry.install.GLOBAL_KEY_DIR + '/id_rsa')
+                os.chmod(ferry.install.GLOBAL_KEY_DIR + '/id_rsa', 0400)
+            except OSError as e:
+                if e.errno != errno.EEXIST:
+                    logging.error("Could not create private key (%s)" % os.strerror(e.errno))
+
+            ferry.install._touch_file(ferry.install.DEFAULT_DOCKER_KEY, 
+                                      ferry.install.GLOBAL_KEY_DIR)
+            logging.warning("Copied private key to " + ferry.install.GLOBAL_KEY_DIR)
+
     """
     This is the command dispatch table. 
     """
     def dispatch_cmd(self, cmd, args, options):
         if(cmd == 'start'):
+            # Check if the user is using the global key directory.
+            # If so, we need to make a copy of the key. 
+            self._check_ssh_key()
+
             arg = args.pop(0)
+            json_arg = {}
             if os.path.exists(arg):
                 json_string = self._read_file_arg(arg)
                 json_arg = json.loads(json_string)
+                json_arg['_file'] = arg
+                json_arg['_file_path'] = arg
             else:
                 # Check if the user wants to use one of the global plans.
-                global_path = FERRY_HOME + '/plans/' + arg
+                global_path = FERRY_HOME + '/data/plans/' + arg
 
                 # Check if the user has passed in a file extension.
                 # If not go ahead and add one. 
+                file_path = global_path
                 n, e = os.path.splitext(global_path)
                 if e == '':
-                    global_path += '.json'
+                    file_path += '.json'
 
-                if os.path.exists(global_path):
-                    json_string = self._read_file_arg(global_path)
+                if os.path.exists(file_path):
+                    json_string = self._read_file_arg(file_path)
                     json_arg = json.loads(json_string)
-                else:
-                    json_arg = {}
-            json_arg['_file'] = arg
+                    json_arg['_file_path'] = file_path
+                json_arg['_file'] = arg
             return self._create_stack(json_arg, args)
         elif(cmd == 'ps'):
             if len(args) > 0 and args[0] == '-a':
@@ -308,6 +390,11 @@ class CLI(object):
             self.installer.start_web()
             return 'started ferry'
         elif(cmd == 'ssh'):
+            # Check if the user is using the global key directory.
+            # If so, we need to make a copy of the key. 
+            self._check_ssh_key()
+
+            # Go ahead and connect to the stack. 
             stack_id = args[0]
             connector_id = None
             if len(args) > 1:
@@ -315,6 +402,7 @@ class CLI(object):
 
             return self._connect_stack(stack_id, connector_id)
         elif(cmd == 'quit'):
+            self._stop_all()
             self.installer.stop_web()
             self.installer._stop_docker_daemon()
             return 'stopped ferry'
@@ -335,8 +423,7 @@ def main(argv=None):
     console = logging.StreamHandler(stream=sys.stderr)
     console.setFormatter(logging.Formatter(fmt='%(asctime)s %(message)s', 
                                            datefmt='%m/%d/%Y %I:%M:%S %p'))
-    # console.setLevel(logging.ERROR)
-    console.setLevel(logging.INFO)
+    console.setLevel(logging.WARNING)
     root_logger = logging.getLogger()
     root_logger.addHandler(console)
     root_logger.setLevel(logging.DEBUG)
@@ -351,7 +438,6 @@ def main(argv=None):
 
             # Execute the commands
             all_cmds = cli.cmds.get_cmds()
-
             if len(all_cmds) > 0:
                 for c in all_cmds.keys():
                     msg = cli.dispatch_cmd(c, all_cmds[c], options)
