@@ -265,7 +265,11 @@ class DockerManager(object):
     def query_stacks(self, constraints=None):
         json_reply = {}
 
-        values = self.cluster_collection.find()
+        if constraints:
+            values = self.cluster_collection.find(constraints)
+        else:
+            values = self.cluster_collection.find()
+
         for v in values:
             time = ''
             s = self.snapshot_collection.find_one( {'snapshot_uuid':v['snapshot_uuid']} )
@@ -288,17 +292,18 @@ class DockerManager(object):
     def query_deployed(self, conf=None):
         json_reply = {}
 
-        values = self.deploy.find(conf=conf)
-        for v in values:
-            time = v['ts'].strftime("%m/%w/%Y (%I:%M %p)")
-            c = self.cluster_collection.find_one( {'uuid':v['cluster_uuid']} )
+        cursors = self.deploy.find(conf=conf)
+        for c in cursors:
+            for v in c:
+                time = v['ts'].strftime("%m/%w/%Y (%I:%M %p)")
+                c = self.cluster_collection.find_one( {'uuid':v['cluster_uuid']} )
                 
-            json_reply[v['uuid']] = { 'uuid' : v['uuid'],
-                                      'base' : c['base'], 
-                                      'ts' : time,
-                                      'backends' : c['backends']['uuids'],
-                                      'connectors': c['connectors'],
-                                      'status': 'deployed' }
+                json_reply[v['uuid']] = { 'uuid' : v['uuid'],
+                                          'base' : c['base'], 
+                                          'ts' : time,
+                                          'backends' : c['backends']['uuids'],
+                                          'connectors': c['connectors'],
+                                          'status': 'deployed' }
         return json.dumps(json_reply, 
                           sort_keys=True,
                           indent=2,
@@ -849,7 +854,7 @@ class DockerManager(object):
     Get the default deployment conf file. 
     """
     def _get_default_conf(self):        
-        return FERRY_HOME + '/conf/deploy_default.json'
+        return FERRY_HOME + '/data/conf/deploy_default.json'
 
     """
     Get the deployment configuration parameters. 
